package index;

import window.Fingerprint;

/**
 * A bean containing the value and the frequency of one one fingerprint.
 * We always have them as a list which is assigned to a document or a finger print.
 */
public class Term {
	/**
	 * Value of the fingerprint
	 */
	Integer id;
	
	/**
	 * The frequency of it in the meant document
	 */
	float frequency;
	
	public Term(Fingerprint fp) {
		this.id = new Integer( fp.getValue());
		this.frequency = 1;
	}
	
	public Term(Integer id, float frequency) {
		this.id = id;
		this.frequency = frequency;
	}
	
	/**
	 * Merging the two terms and obtaining the mean value used to obtain the
	 * centroid of the merged clusters. In the computation of centroid the size of each
	 * cluster is important to let the bigger cluster have more effect.
	 * 
	 * @param other the other term to be merged with
	 * @param first the weight of the first cluster => number of documents
	 * @param second the weight of the second cluster => number of documents
	 * @throws TermIDConflict the terms must have similar IDs
	 */
	public void merge(Term other, int first, int second) throws TermIDConflict {
		if( other == null) {
			float temp = frequency * first;
			temp /= (first + second);
			
			frequency = temp;
		} else {
			if( !id.equals(other.id))
				throw new TermIDConflict();
			
			float temp = frequency * first;
			temp += other.frequency * second;
			temp /= (first + second);
			
			frequency = temp;
		}
	}

	public Integer getId() {
		return id;
	}

	public float getFrequency() {
		return frequency;
	}
	
	public void incFrequency() {
		frequency++;
	}
	
	public boolean equals(Object arg0) {
		return id.equals(((Term)arg0).id);
	}

	public int hashCode() {
		return id.hashCode();
	}

	public String toString() {
		return "ID: " + id + " Freq: " + frequency;
	}
	
//	public Object clone() {
//		Term retval = new Term(id, frequency);
//		return retval;
//	}
}
