package similarity;

import index.Term;
import index.Thesaurus;

import java.util.Iterator;
import java.util.List;

/**
 * The template class for implementing Vector Space Method Similarity
 */
public abstract class VSM {
	
	/**
	 * The number of documents in the collection
	 */
	protected int N;
	
	/**
	 * The collection to be queried
	 */
	protected Thesaurus thesauri;
	
	public VSM(Thesaurus thesauri) {
		this.thesauri = thesauri;
		N = thesauri.getNumberOfDocuments();		
	}
	
	/**
	 * Compute the similarity of two documents. Because of float precission(0 is not realy 0) the value may
	 * be a little more than zero.
	 * @param d1 first documetn
	 * @param d2 second document
	 * @return the similarity in [0, 1]
	 */
	public double similarity(List d1, List d2) {
		//Border conditions
		double sigma = 0;

		for (Iterator it = d1.iterator(); it.hasNext();) {
			Term term = (Term) it.next();
			
			if(d2.contains(term))
				sigma += firstValue(term) * secondValue(term);
		}
		
		sigma /= firstNorm(d1) * secondNorm(d2);

		return sigma;
	}

	/**
	 * The tf * idf according to scheme for the first document; returning one element of Sigma
	 * @param term the for which the value will be computed
	 * @return one element of Sigma
	 */
	protected abstract double firstValue(Term term);

	/**
	 * The tf * idf according to scheme for the second document; returning one element of Sigma
	 * @param term the for which the value will be computed
	 * @return one element of Sigma
	 */
	protected abstract double secondValue(Term term);

	/**
	 * The normalization for the first document
	 * @param d1 the document to be normalized
	 * @return normalization value of document
	 */
	protected abstract double firstNorm(List d1);

	/**
	 * The normalization for the second document
	 * @param d2 the document to be normalized
	 * @return normalization value of document
	 */
	protected abstract double secondNorm(List d2);
}
